@@ -1,9 +1,7 @@
 #!/usr/bin/python3
-from debian.changelog import Version
-
 from lintian_brush.control import (
     drop_dependency,
-    ensure_minimum_version,
+    ensure_minimum_debhelper_version,
     update_control,
     )
 
@@ -14,9 +12,8 @@ def bump_debhelper(control):
         control["Build-Depends"],
         "dh-systemd")
     if old_build_depends != control["Build-Depends"]:
-        control["Build-Depends"] = ensure_minimum_version(
-            control["Build-Depends"],
-            "debhelper", Version("9.20160709"))
+        control["Build-Depends"] = ensure_minimum_debhelper_version(
+            control["Build-Depends"], "9.20160709")
 
 
 if update_control(source_package_cb=bump_debhelper):
