@@ -19,6 +19,7 @@
 
 from debian.changelog import Changelog
 import errno
+import io
 import os
 import re
 import shutil
@@ -26,6 +27,7 @@ import subprocess
 import sys
 import tempfile
 import time
+import traceback
 import warnings
 
 from breezy import ui
@@ -36,6 +38,7 @@ from breezy.clean_tree import (
     iter_deletables,
     )
 from breezy.commit import NullCommitReporter
+from breezy.errors import NoSuchFile
 from breezy.rename_map import RenameMap
 from breezy.trace import note
 from breezy.transform import revert
@@ -43,10 +46,11 @@ from breezy.transform import revert
 from debian.deb822 import Deb822
 
 
-__version__ = (0, 18)
+__version__ = (0, 20)
 version_string = '.'.join(map(str, __version__))
 SUPPORTED_CERTAINTIES = ['certain', 'possible', None]
 DEFAULT_MINIMUM_CERTAINTY = 'certain'
+USER_AGENT = 'lintian-brush/' + version_string
 
 
 class NoChanges(Exception):
@@ -77,7 +81,7 @@ class FixerScriptFailed(FixerFailed):
     def __str__(self):
         return ("Script %s failed with exit code: %d\n%s\n" % (
                 self.path, self.returncode,
-                self.errors.decode(errors='replace')))
+                self.errors))
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
@@ -99,13 +103,6 @@ class NotDebianPackage(Exception):
         super(NotDebianPackage, self).__init__(tree.basedir)
 
 
-class GeneratedControlFile(Exception):
-    """The specified file is generated."""
-
-    def __init__(self, path):
-        self.path = path
-
-
 class PendingChanges(Exception):
     """The directory has pending changes."""
 
@@ -164,6 +161,92 @@ class Fixer(object):
         raise NotImplementedError(self.run)
 
 
+def parse_script_fixer_output(text):
+    """Parse the output from a script fixer."""
+    lines = []
+    fixed_tags = []
+    certainty = None
+    for line in text.splitlines():
+        # TODO(jelmer): Do this in a slighly less hackish manner
+        try:
+            (key, value) = line.split(':', 1)
+        except ValueError:
+            lines.append(line)
+        else:
+            if key == 'Fixed-Lintian-Tags':
+                fixed_tags = value.strip().split(',')
+            elif key == 'Certainty':
+                certainty = value.strip()
+            else:
+                lines.append(line)
+    if certainty not in SUPPORTED_CERTAINTIES:
+        raise UnsupportedCertainty(certainty)
+    return FixerResult('\n'.join(lines), fixed_tags, certainty)
+
+
+class PythonScriptFixer(Fixer):
+    """A fixer that is implemented as a python script.
+
+    This gets used just for Python scripts, and significantly speeds
+    things up because it prevents starting a new Python interpreter
+    for every fixer.
+    """
+
+    def __init__(self, name, lintian_tags, script_path):
+        super(PythonScriptFixer, self).__init__(name, lintian_tags)
+        self.script_path = script_path
+
+    def __repr__(self):
+        return "<%s(%r)>" % (self.__class__.__name__, self.name)
+
+    def run(self, basedir, current_version, compat_release,
+            minimum_certainty=DEFAULT_MINIMUM_CERTAINTY,
+            trust_package=False, allow_reformatting=False):
+        env = dict(os.environ.items())
+        env['CURRENT_VERSION'] = str(current_version)
+        env['COMPAT_RELEASE'] = compat_release
+        env['MINIMUM_CERTAINTY'] = minimum_certainty
+        env['TRUST_PACKAGE'] = 'true' if trust_package else 'false'
+        env['REFORMATTING'] = ('allow' if allow_reformatting else 'disallow')
+        try:
+            old_env = os.environ
+            old_stderr = sys.stderr
+            old_stdout = sys.stdout
+            sys.stderr = io.StringIO()
+            sys.stdout = io.StringIO()
+            os.environ = env
+            old_cwd = os.getcwd()
+            try:
+                os.chdir(basedir)
+                with open(self.script_path, 'r') as f:
+                    code = compile(f.read(), self.script_path, 'exec')
+                    exec(code, {})
+            except SystemExit as e:
+                retcode = e.code
+            except BaseException as e:
+                traceback.print_exception(
+                    type(e), e, e.__traceback__, file=sys.stderr)
+                raise FixerScriptFailed(
+                    self.script_path, 1,
+                    sys.stderr.getvalue())
+            else:
+                retcode = 0
+            description = sys.stdout.getvalue()
+            err = sys.stderr.getvalue()
+        finally:
+            os.environ = old_env
+            sys.stderr = old_stderr
+            sys.stdout = old_stdout
+            os.chdir(old_cwd)
+
+        if retcode == 2:
+            raise NoChanges()
+        if retcode != 0:
+            raise FixerScriptFailed(self.script_path, retcode, err)
+
+        return parse_script_fixer_output(description)
+
+
 class ScriptFixer(Fixer):
     """A fixer that is implemented as a shell/python/etc script."""
 
@@ -194,27 +277,8 @@ class ScriptFixer(Fixer):
                 stderr.seek(0)
                 raise FixerScriptFailed(
                         self.script_path, p.returncode,
-                        stderr.read())
-        description = description.decode('utf-8')
-        lines = []
-        fixed_tags = []
-        certainty = None
-        for line in description.splitlines():
-            # TODO(jelmer): Do this in a slighly less hackish manner
-            try:
-                (key, value) = line.split(':', 1)
-            except ValueError:
-                lines.append(line)
-            else:
-                if key == 'Fixed-Lintian-Tags':
-                    fixed_tags = value.strip().split(',')
-                elif key == 'Certainty':
-                    certainty = value.strip()
-                else:
-                    lines.append(line)
-        if certainty not in SUPPORTED_CERTAINTIES:
-            raise UnsupportedCertainty(certainty)
-        return FixerResult('\n'.join(lines), fixed_tags, certainty)
+                        stderr.read().decode('utf-8', 'replace'))
+        return parse_script_fixer_output(description.decode('utf-8'))
 
 
 def find_fixers_dir():
@@ -250,7 +314,10 @@ def read_desc_file(path):
                         for tag in paragraph['Lintian-Tags'].split(',')]
             else:
                 tags = []
-            yield ScriptFixer(name, tags, script_path)
+            if script_path.endswith('.py'):
+                yield PythonScriptFixer(name, tags, script_path)
+            else:
+                yield ScriptFixer(name, tags, script_path)
 
 
 def available_lintian_fixers(fixers_dir=None):
@@ -379,14 +446,17 @@ def only_changes_last_changelog_block(tree):
         return str(new_cl) == str(old_cl)
 
 
-def reset_tree(local_tree):
+def reset_tree(local_tree, dirty_tracker=None):
     """Reset a tree back to its basis tree.
 
     This will leave ignored and detritus files alone.
 
     Args:
       local_tree: tree to work on
+      dirty_tracker: Optional dirty tracker
     """
+    if dirty_tracker and not dirty_tracker.is_dirty():
+        return
     revert(local_tree, local_tree.branch.basis_tree(), None)
     deletables = list(iter_deletables(
         local_tree, unknown=True, ignored=False, detritus=False))
@@ -407,10 +477,25 @@ def certainty_sufficient(actual_certainty, minimum_certainty):
     return True
 
 
+def check_clean_tree(local_tree):
+    """Check that a tree is clean and has no pending changes or unknown files.
+
+    Args:
+      local_tree: The tree to check
+    Raises:
+      PendingChanges: When there are pending changes
+    """
+    # Just check there are no changes to begin with
+    if local_tree.has_changes():
+        raise PendingChanges(local_tree)
+    if list(local_tree.unknowns()):
+        raise PendingChanges(local_tree)
+
+
 def run_lintian_fixer(local_tree, fixer, committer=None,
                       update_changelog=None, compat_release=None,
                       minimum_certainty=None, trust_package=False,
-                      allow_reformatting=False):
+                      allow_reformatting=False, dirty_tracker=None):
     """Run a lintian fixer on a tree.
 
     Args:
@@ -424,18 +509,16 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
         about its changes.
       trust_package: Whether to run code from the package if necessary
       allow_reformatting: Whether to allow reformatting of changed files
+      dirty_tracker: Optional object that can be used to tell if the tree
+        has been changed.
     Returns:
       tuple with set of FixerResult, summary of the changes
     """
-    # Just check there are no changes to begin with
-    if local_tree.has_changes():
-        raise PendingChanges(local_tree)
-    if list(local_tree.unknowns()):
-        raise PendingChanges(local_tree)
-    if not local_tree.has_filename('debian/changelog'):
+    try:
+        with local_tree.get_file('debian/changelog') as f:
+            cl = Changelog(f, max_blocks=1)
+    except NoSuchFile:
         raise NotDebianPackage(local_tree)
-    with local_tree.get_file('debian/changelog') as f:
-        cl = Changelog(f, max_blocks=1)
     if cl.distributions == 'UNRELEASED':
         current_version = cl.version
     else:
@@ -453,18 +536,38 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
             trust_package=trust_package,
             allow_reformatting=allow_reformatting)
     except BaseException:
-        reset_tree(local_tree)
+        reset_tree(local_tree, dirty_tracker)
         raise
     if not certainty_sufficient(result.certainty, minimum_certainty):
-        reset_tree(local_tree)
-        raise NoChanges("Script didn't make any changes")
-    local_tree.smart_add([local_tree.basedir])
+        reset_tree(local_tree, dirty_tracker)
+        raise NoChanges("Certainty of script's changes not high enough")
+    if dirty_tracker:
+        relpaths = dirty_tracker.relpaths()
+        local_tree.add(
+            [p for p in relpaths
+             if local_tree.has_filename(p) and not
+                local_tree.is_ignored(p)])
+        specific_files = [
+            p for p in relpaths
+            if local_tree.is_versioned(p)]
+        if not specific_files:
+            raise NoChanges("Script didn't make any changes")
+    else:
+        local_tree.smart_add([local_tree.basedir])
+        specific_files = None
+
+    basis_tree = local_tree.basis_tree()
     if local_tree.supports_setting_file_ids():
         RenameMap.guess_renames(
-            local_tree.basis_tree(), local_tree, dry_run=False)
+            basis_tree, local_tree, dry_run=False)
 
-    if not local_tree.has_changes():
-        raise NoChanges("Script didn't make any changes")
+    if len(local_tree.get_parent_ids()) <= 1:
+        changes = local_tree.iter_changes(
+            basis_tree, specific_files=specific_files)
+        try:
+            next(changes)
+        except StopIteration:
+            raise NoChanges("Script didn't make any changes")
 
     if not result.description:
         raise DescriptionMissing()
@@ -482,6 +585,8 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
     if update_changelog:
         subprocess.check_call(
             ["dch", "--no-auto-nmu", summary], cwd=local_tree.basedir)
+        if specific_files:
+            specific_files.append('debian/changelog')
 
     description = result.description
     for tag in result.fixed_lintian_tags:
@@ -496,7 +601,8 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
 
     local_tree.commit(description, allow_pointless=False,
                       reporter=NullCommitReporter(),
-                      committer=committer)
+                      committer=committer,
+                      specific_files=specific_files)
     # TODO(jelmer): Run sbuild & verify lintian warning is gone?
     return result, summary
 
@@ -504,7 +610,8 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
 def run_lintian_fixers(local_tree, fixers, update_changelog=True,
                        verbose=False, committer=None,
                        compat_release=None, minimum_certainty=None,
-                       trust_package=False, allow_reformatting=False):
+                       trust_package=False, allow_reformatting=False,
+                       use_inotify=None):
     """Run a set of lintian fixers on a tree.
 
     Args:
@@ -519,6 +626,8 @@ def run_lintian_fixers(local_tree, fixers, update_changelog=True,
         about its changes.
       trust_package: Whether to run code from the package if necessary
       allow_reformatting: Whether to allow reformatting of changed files
+      use_inotify: Use inotify to watch changes (significantly improves
+        performance). Defaults to None (automatic)
     Returns:
       Tuple with two lists:
         1. list of tuples with (lintian-tag, certainty, description) of fixers
@@ -526,23 +635,37 @@ def run_lintian_fixers(local_tree, fixers, update_changelog=True,
         2. dictionary mapping fixer names for fixers that failed to run to the
            error that occurred
     """
-    if local_tree.has_filename('debian/control.in'):
-        raise GeneratedControlFile('debian/control.in')
+    check_clean_tree(local_tree)
     failed_fixers = {}
     fixers = list(fixers)
     ret = []
+    if use_inotify is True:
+        from .dirty_tracker import DirtyTracker
+        dirty_tracker = DirtyTracker(local_tree)
+    elif use_inotify is False:
+        dirty_tracker = None
+    else:
+        try:
+            from .dirty_tracker import DirtyTracker
+        except ImportError:
+            dirty_tracker = None
+        else:
+            dirty_tracker = DirtyTracker(local_tree)
     with ui.ui_factory.nested_progress_bar() as pb:
         for i, fixer in enumerate(fixers):
             pb.update('Running fixer %r on %s' % (fixer, local_tree.basedir),
                       i, len(fixers))
             start = time.time()
+            if dirty_tracker:
+                dirty_tracker.mark_clean()
             try:
                 result, summary = run_lintian_fixer(
                         local_tree, fixer, update_changelog=update_changelog,
                         committer=committer, compat_release=compat_release,
                         minimum_certainty=minimum_certainty,
                         trust_package=trust_package,
-                        allow_reformatting=allow_reformatting)
+                        allow_reformatting=allow_reformatting,
+                        dirty_tracker=dirty_tracker)
             except FixerFailed as e:
                 failed_fixers[fixer.name] = e
                 if verbose:
