@@ -22,10 +22,12 @@ def test_suite():
     names = [
         'control',
         'copyright',
+        'dirty_tracker',
         'lintian_overrides',
         'reformatting',
         'rules',
         'run',
+        'salsa',
         'systemd',
         ]
     module_names = [__name__ + '.test_' + name for name in names]
